/***************************************************************************
 *   Copyright (C) 2007 by Jan Vermeulen                                   *
 *   janverm@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <math.h>
#include "liquid.h"

static SDL_Surface* liquid_effect = NULL;
static SDL_Surface* image = NULL;
static int inited;
static float anim = 0.0;
static int temp;

int fx_liquid_init(unsigned long val)
{
	SDL_Surface *screen = (SDL_Surface *) val;
	SDL_Surface *s;

	if (inited) return 0;

	liquid_effect = SDL_CreateRGBSurface(SDL_SWSURFACE,
					     RES_X, RES_Y, 16,
					     screen->format->Rmask,
					     screen->format->Gmask,
					     screen->format->Bmask,
					     screen->format->Amask);
#if (RES_X == 480)
	s = SDL_LoadBMP("./pix/chamwoman.bmp");
#elif (RES_X == 800)
	s = SDL_LoadBMP("./pix/chamwoman_hi.bmp");
#else
#error Only horizontal resolutions of 480 or 800 are supported.
#endif

	image = SDL_DisplayFormat(s);
	
	if (!image) {
		fprintf(stderr, "Cannot open EIA logo: %s\n", SDL_GetError());
		return -1;
	}

	SDL_FreeSurface(s);
	inited = 1;
	return 0;
}

int fx_liquid_exit(unsigned long val)
{
	temp = val;

	if(liquid_effect) {
		SDL_FreeSurface(liquid_effect);
	}
	if(image) {
		SDL_FreeSurface(image);
	}
	return 0;
}

SDL_Surface* fx_liquid_play(unsigned long val)
{
	int x,y;
	SDL_Rect src,dst;
	
	temp = val;

	SDL_FillRect(liquid_effect, NULL, 0);
	anim = anim + 0.1;
	for (x=0;x<RES_X;x+=20) {
		src.x = x;
		dst.x = (x+(sin(anim + x * 0.01) * 10)) + 5;
		for (y=0;y<RES_Y;y+=20) {
			src.y = y;
			dst.y = (y + (sin(anim + y * 0.01) * 10)) + 5;
			dst.w = dst.h = src.w = src.h = 20;
			SDL_BlitSurface(image,&src,liquid_effect,&dst);
		}
	}
	return liquid_effect;
}

struct demopart_t fx_liquid = {
	.time = 5,
	.init = fx_liquid_init,
	.exit = fx_liquid_exit,
	.play = fx_liquid_play,
	.info = "It's not solid!",
};

