/***************************************************************************
 *   Copyright (C) 2007 by Jan Vermeulen                                   *
 *   janverm@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <math.h>
#include "liquid.h"

static SDL_Surface *rotozoom_effect = NULL;
static SDL_Surface *tile = NULL;
static int roto[256];
static int roto2[256];
static int path = 0, zpath = 0;
static int inited = 0;
static int temp;

static void draw_tile(int stepx, int stepy, int zoom)
{
	Uint16 *image = (Uint16 *) rotozoom_effect->pixels;
	Uint16 *texture = (Uint16 *) tile->pixels;
	int x, y, i, j, xd, yd, a, b, sx, sy;

	sx = sy = 0;
	xd = (stepx * zoom) >> 12;
	yd = (stepy * zoom) >> 12;

	/* Stepping across and down the screen, each screen row has a
	 * starting coordinate in the texture: (sx, sy).  As each screen
	 * row is traversed, the current texture coordinate (x, y) is
	 * modified by (xd, yd), which are (sin(rot), cos(rot)) multiplied
	 * by the current zoom factor.  For each vertical step, (xd, yd)
	 * is rotated 90 degrees, to become (-yd, xd).
	 * 
	 * More fun can be had by playing around with x, y, xd, and yd as
	 * you move about the image.
	 */

	for (j = 0; j < RES_Y; j++) {
		x = sx;
		y = sy;
		for (i = 0; i < RES_X; i++) {
			a = x >> 12 & 255;
			b = y >> 12 & 255;
			*image++ = texture[b * tile->w + a];
			x += xd;
			y += yd;
		}
		sx -= yd;
		sy += xd;
	}
}

int fx_rotozoom_init(unsigned long val)
{
	SDL_Surface *screen = (SDL_Surface *) val;
	SDL_Surface *s;
	int i;

	if (inited) return 0;

	rotozoom_effect = SDL_CreateRGBSurface(SDL_SWSURFACE, 
					       RES_X, RES_Y, 16,
					       screen->format->Rmask,
					       screen->format->Gmask,
					       screen->format->Bmask,
					       screen->format->Amask);

	/* load the background image and make it the same as
	 * our display properties. */
	s = SDL_LoadBMP("./pix/EIA_logo_256x256.bmp");
	tile = SDL_DisplayFormat(s);

	if (!tile) {
		fprintf(stderr, "Cannot open EIA logo: %s\n", SDL_GetError());
		return -1;
	}

	if (tile->w != 256 || tile->h != 256) {
		fprintf(stderr, "Tile dimensions must be 256x256!\n");
		return -1;
	}

	for (i = 0; i < 256; i++) {
		float rad = (float)i * 1.41176 * 0.0174532;
		float c = cos(rad);
		roto[i] = (c + 0.8) * 4096.0;
		roto2[i] = (14.5 * c) * 4096.0;
	}
	
	SDL_FreeSurface(s);

	inited = 1;
	return 0;
}

int fx_rotozoom_exit(unsigned long val)
{
	temp = val;
	if(rotozoom_effect) {
		SDL_FreeSurface(rotozoom_effect);
	}
	if(tile) {
		SDL_FreeSurface(tile);
	}
	return 0;
}

SDL_Surface *fx_rotozoom_play(unsigned long val)
{
	temp = val;
	/* draw the tile at current zoom and rotation */
	draw_tile(roto[path], roto[(path + 128) & 255], roto2[zpath]);

	path = (path - 1) & 255;
	zpath = (zpath + 1) & 255;
	SDL_Delay(20);	
	return rotozoom_effect;
}

struct demopart_t fx_rotozoom = {
	.time = 8,
	.init = fx_rotozoom_init,
	.exit = fx_rotozoom_exit,
	.play = fx_rotozoom_play,
	.info = "Rotozoomer!",
};
