/***************************************************************************
 *   Copyright (C) 2007 by Jan Vermeulen                                   *
 *   janverm@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "liquid.h"

static const short amplitudes[4] = { -250, -425, -350, -650 };
static const Uint16 b = RES_X - 10;
static const Uint16 c = RES_X * 10;
static const Uint32 d = (RES_X * RES_Y) - (RES_X * 10);
static const int maxarray = RES_X * RES_Y;

static SDL_Surface *water_effect = NULL;
static SDL_Surface *image = NULL;
static short wavemap[RES_X * RES_Y];
static short old_wavemap[RES_X * RES_Y];
static short *oldptr = old_wavemap;
static short *newptr = wavemap;
static Uint8 amp_index = 0;
static short *address_new, *address_old, *temp;
static short height, xdiff;
static Uint16 *pwater, *pimage;
static int inited = 0;
static unsigned long tmp;

static void new_raindrop(void)
{
	Uint32 v, w;

	/* try until within screen */
	do {
		v = (int)(maxarray * (rand() / (RAND_MAX + 1.0)));
		w = v % RES_X;
	}
	while (w < 10 || w > b || v < c || v > d);

	/* variate between 4 different waves */
	wavemap[v] = amplitudes[amp_index++];
	amp_index &= 4;
}

int fx_water_init(unsigned long val)
{
	SDL_Surface *s;
	int i;
	SDL_Surface *screen = (SDL_Surface *) val;

	if (inited) return 0;
	
	water_effect = SDL_CreateRGBSurface(SDL_SWSURFACE,
					    RES_X, RES_Y, 16,
					    screen->format->Rmask,
					    screen->format->Gmask,
					    screen->format->Bmask,
					    screen->format->Amask);
#if (RES_X == 480)
	s = SDL_LoadBMP("./pix/knikkers01.bmp");
#elif (RES_X == 800)
	s = SDL_LoadBMP("./pix/knikkers01_hi.bmp");
#else
#error Only horizontal resolutions of 480 or 800 are supported.
#endif

	image = SDL_DisplayFormat(s);
	if (!image) {
		fprintf(stderr, "Cannot open file knikkers01.bmp: %s\n",
			SDL_GetError());
		return -1;
	}

	/* init wavemaps */
	for (i = 0; i < maxarray; ++i) {
		wavemap[i] = 0;
		old_wavemap[i] = 0;
	}

	SDL_FreeSurface(s);
	inited = 1;
	return 0;
}

int fx_water_exit(unsigned long val)
{
	tmp = val;
	SDL_FreeSurface(image);
	SDL_FreeSurface(water_effect);
	return 0;
}

SDL_Surface *fx_water_play(unsigned long val)
{
	Uint32 i, j, t;
	
	new_raindrop();

	tmp = val;

	/* compute waves */
	t = RES_X + 1;
	address_new = newptr + t;
	address_old = oldptr + t;

	for (i = 1; i < RES_Y - 1; ++i) {
		for (j = 1; j < RES_X - 1; ++j) {
			height = 0;
			height += *(address_new + RES_X);
			height += *(address_new - 1);
			height += *(address_new + 1);
			height += *(address_new - RES_X);
			height >>= 1;
			height -= *address_old;
			height -= height >> 5;
			*address_old = height;
			address_new++;
			address_old++;
		}
		address_new += 2;	/* next scanline starting at pos 1 */
		address_old += 2;
	}

	t = RES_X + 1;
	address_old = oldptr + t;
	pwater = (Uint16 *) water_effect->pixels + t;
	pimage = (Uint16 *) image->pixels + t;

	/* draw waves */
	for (i = 1; i < RES_Y - 1; ++i) {
		for (j = 1; j < RES_X - 1; ++j) {
			xdiff = *(address_old + 1) - *(address_old);
			*pwater = *(pimage + xdiff);
			address_old++;
			pwater++;
			pimage++;
		}
		address_old += 2;
		pwater += 2;	/* next scanline starting at pos 1 */
		pimage += 2;
	}

	/* swap tables for next iteration */
	temp = newptr;
	newptr = oldptr;
	oldptr = temp;

	return water_effect;
}

struct demopart_t fx_water = {
	.time = 10,
	.init = fx_water_init,
	.exit = fx_water_exit,
	.play = fx_water_play,
	.info = "Raindrops!",
};
