/***************************************************************************
 *   Copyright (C) 2007 by Jan Vermeulen                                   *
 *   janverm@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <string.h>
#include "gui.h"
#include "cybunit.h"
#include "charsets.h"

SDL_Color selection[40];

void WrChar(SDL_Surface * screen,
	    unsigned char kx, unsigned char ky,
	    unsigned char kar,
	    Uint8 r, Uint8 g, Uint8 b, unsigned char charsetnr)
{
	unsigned int offset, xpxl, ypxl;
	unsigned char chb, i, j, set, rstep, gstep, bstep;

	/* calc steps for colours to lighten up */
	rstep = r >> 4;
	gstep = g >> 4;
	bstep = b >> 4;
	r = 0;
	g = 0;
	b = 0;

	/* vertical width of a character is 16 pixels
	   horizontal width is 8 pixels */
	offset = (charsetnr << 12) + (kar << 4);
	ypxl = ky << 4;
	xpxl = kx << 3;

	for (i = 0; i < 16; i++) {
		chb = charsets[offset + i];
		for (j = 0; j < 8; j++) {
			set = chb & (0x80 >> j);
			if (set != 0)
				putpixel(screen, xpxl + j, ypxl + i, r, g, b);
		}
		r += rstep;
		g += gstep;
		b += bstep;
	}
}

void WrStr(SDL_Surface * screen,
	   unsigned char mx, unsigned char my,
	   const char *text, Uint8 r, Uint8 g, Uint8 b, unsigned char charsetnr)
{
	unsigned int offset, xpxl, ypxl;
	unsigned char chb, i, j, s, set;

	for (s = 0; s < strlen(text); s++) {
		offset = (charsetnr << 12) + (text[s] << 4);
		ypxl = (my << 4);
		xpxl = ((mx + s) << 3);
		for (i = 0; i < 16; i++) {
			chb = charsets[offset + i];
			for (j = 0; j < 8; j++) {
				set = chb & (0x80 >> j);
				if (set != 0)
					putpixel(screen, (xpxl + j), (ypxl + i),
						 r, g, b);
			}
		}
	}
}

void DrawPipe(SDL_Surface * screen, int y, SDL_Color color)
/* This function draws a colored pipe at Y-position with a basic color
   and shades it.
   Light shade on top, darker shade at bottom */
{
	unsigned char i, r, g, b, r2, g2, b2;
	r = r2 = color.r;
	g = g2 = color.g;
	b = b2 = color.b;

	for (i = 0; i < 5; i++) {
		Hline(screen, y + i + 6, r, g, b);
		Hline(screen, y - i + 5, r2, g2, b2);
		if (r > 24 ) 
			r -= 24;
		if (g > 24 )
			g -= 24;
		if (b > 24 )
			b -= 24;

		if (r2 > 8 )
			r2 -= 8;
		if (g2 > 8 )
			g2 -= 8;
		if (b2 > 8 )
			b2 -= 8;
	}
}

void DrawBox(SDL_Surface * area, unsigned int x, unsigned int y,
	     unsigned int length)
{
	static int i = 20;
	static int idir = 1;

	unsigned int xx, yy;
	SDL_Color checkcolor;

	/* Selectionbar colors */
	for (xx = 0; xx < 40; xx++) {
		selection[xx].r = xx * 4 + 10;
		selection[xx].g = 0;
		selection[xx].b = xx * 4 + 10;
	}

	for (yy = 0; yy < 16; yy++)
		for (xx = 0; xx < (length << 3); xx++) {
			checkcolor =
			    getpixel(area, (x << 3) + xx, (y << 4) + yy);
			if ((checkcolor.r + checkcolor.g + checkcolor.b) == 0)
				putpixel(area, (x << 3) + xx, (y << 4) + yy,
					 selection[i].r, selection[i].g,
					 selection[i].b);
		}
	i += idir;
	if ((i > 38) || (i < 15))
		idir = -idir;
}
