/***************************************************************************
 *   Copyright (C) 2009 by Jan Vermeulen                                   *
 *   janverm@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include "cybunit.h"
#include "gui.h"
#include "liquid.h"

/* demoparts */
struct demopart_t *demopart[MAX_DEMOPARTS] = {
	&fx_rotozoom,
	&fx_lens,
	&fx_water,
	&fx_bump,
	&fx_liquid,
};

/* basic surfaces */
SDL_Surface *screen;
SDL_Surface *charbuffer;
SDL_Surface *infopane;

/* global vars */
SDL_Rect src, dst;
int current_part = 0;
int old_time = 0;

/* ----------------------------------------------------------------------- */

void clean_quit(void)
{
	int i;
	SDL_FreeSurface(infopane);
	SDL_FreeSurface(screen);
	SDL_FreeSurface(charbuffer);
	for (i = 0; i < MAX_DEMOPARTS; i++)
		demopart[i]->exit(i);
	SDL_Quit();
}

/* ----------------------------------------------------------------------- */

void create_main_surfaces(void)
{
	int i, j;
	SDL_Surface *s;

	/* main window */
	screen = MakeWindow(RES_X, RES_Y, 16);

	/* info pane */
	s = SDL_CreateRGBSurface(SDL_SWSURFACE,120,60,16,
				 screen->format->Rmask, screen->format->Gmask,
				 screen->format->Bmask, screen->format->Amask);
	SDL_SetAlpha(s, SDL_SRCALPHA, 128);
	infopane = SDL_DisplayFormat(s);

	/* all charsets, pre-rendered */
	s = SDL_CreateRGBSurface(SDL_SWSURFACE, 1024, 400, 16,
				 screen->format->Rmask, screen->format->Gmask,
				 screen->format->Bmask, screen->format->Amask);
	charbuffer = SDL_DisplayFormat(s);
	for (j = 0; j < 9; j++)
		for (i = 0; i < 256; i++)
			WrChar(charbuffer, ((j * 256) + i) % 128,
			       ((j * 256) + i) >> 7, i, 160, i, 210 + j, j);

	SDL_FreeSurface(s);
}

/* ----------------------------------------------------------------------- */

int do_events(void)
{
	int temp;
	SDL_Event event;

	temp = 0;

	if (SDL_PollEvent(&event) >= 0)
		switch (event.type) {
		case SDL_QUIT:
			temp = 1;
			break;
		case SDL_KEYDOWN:
			if ((event.key.keysym.sym == SDLK_ESCAPE) ||
			    (event.key.keysym.sym == SDLK_SPACE))
				temp = 1;
			break;
		}
	return temp;
}

/* ----------------------------------------------------------------------- */

void draw_effect(int part)
{
	SDL_Surface *s;

	/* Simple blitting with source colour keying.
	 * This can be extended to blit multiple effects on top of eachother 
	 * by passing the surfaces to eachoter.
	 * An API change is in order then in struct demopart_t
	 */
	s = demopart[part]->play(part);
	/*
	src.x = 0;
	src.y = 0;
	src.h = RES_Y;
	src.w = RES_X;
	dst.x = 0;
	dst.y = 0;
	dst.h = RES_Y;
	dst.w = RES_X;
	*/
	SDL_BlitSurface(s, NULL, screen, NULL);
}

/* ----------------------------------------------------------------------- */

int time_passed(int part)
{
	if (part != current_part) {
		/* we switched to a new effect */
		old_time = SDL_GetTicks();
		current_part = part;
	}
	if ((abs(SDL_GetTicks() - old_time) / 1000) > demopart[part]->time)
		return 1;
	else
		return 0;
}

/* ----------------------------------------------------------------------- */

void draw_info(int part)
{
	SDL_Rect dst;
	dst.x=0;dst.y=32;dst.w=120;dst.h=60-32-1;
	SDL_FillRect(infopane, &dst, 0);
	WrStr(infopane,  0, 0, "EiA"           , 20, 20, 255, 8);
	WrStr(infopane,  0, 1, "Electronics NV", 20, 20, 255, 8);
	WrStr(infopane,  0, 2, demopart[part]->info, 20, 20, 255, 9);
	dst.x=10;dst.y=164;dst.w=120;dst.h=60;
	SDL_BlitSurface(infopane,NULL,screen,&dst);
}

/* ----------------------------------------------------------------------- */

int main(void)
{
	int part, finished, i;

	/* do initializations */
	create_main_surfaces();

	/* init fx parts */
	for (i = 0; i < MAX_DEMOPARTS; i++)
		if (demopart[i]->init((unsigned long)screen) < 0)
			clean_quit();

	/* init loop vars */
	finished = 0;
	part = 0;

	/* start timed demo loop */
	while (!finished) {
		draw_effect(part);
		/* draw_info(part); */
		if (time_passed(part)) {
			part++;
			if (part >= MAX_DEMOPARTS)
				part = 0;
		}
		finished = do_events();
#ifdef CONFIG_PC
		SDL_Delay(5);
#warning Adding extra delay!
#endif
		SDL_Flip(screen);
	}

	printf
	    ("You closed the main window, quitting and flushing SDL buffers.\n");
	clean_quit();
	return 0;

}
